# Author: Alexandre Boucaud <aboucaud@lal.in2p3.fr>
# License: BSD 3 clause

from keras.models import Model
from keras.layers import Conv2D, Dropout, Input, concatenate, MaxPooling2D, Conv2DTranspose


def unet_dropout(input_shape, output_channels, dropout):
    inputs = Input(input_shape)
    conv1 = Conv2D(8, (3, 3), activation='relu', padding='same')(inputs)
    conv1 = Dropout(dropout)(conv1)
    conv1 = Conv2D(8, (3, 3), activation='relu', padding='same')(conv1)
    conv1 = Dropout(dropout)(conv1)
    pool1 = MaxPooling2D(pool_size=(2, 2))(conv1)

    conv2 = Conv2D(16, (3, 3), activation='relu', padding='same')(pool1)
    conv2 = Dropout(dropout)(conv2)
    conv2 = Conv2D(16, (3, 3), activation='relu', padding='same')(conv2)
    conv2 = Dropout(dropout)(conv2)
    pool2 = MaxPooling2D(pool_size=(2, 2))(conv2)

    conv3 = Conv2D(32, (3, 3), activation='relu', padding='same')(pool2)
    conv3 = Dropout(dropout)(conv3)
    conv3 = Conv2D(32, (3, 3), activation='relu', padding='same')(conv3)
    conv3 = Dropout(dropout)(conv3)
    pool3 = MaxPooling2D(pool_size=(2, 2))(conv3)

    conv4 = Conv2D(64, (3, 3), activation='relu', padding='same')(pool3)
    conv4 = Dropout(dropout)(conv4)
    conv4 = Conv2D(64, (3, 3), activation='relu', padding='same')(conv4)
    conv4 = Dropout(dropout)(conv4)
    pool4 = MaxPooling2D(pool_size=(2, 2))(conv4)

    conv5 = Conv2D(128, (3, 3), activation='relu', padding='same')(pool4)
    conv5 = Dropout(dropout)(conv5)
    conv5 = Conv2D(128, (3, 3), activation='relu', padding='same')(conv5)
    conv5 = Dropout(dropout)(conv5)

    up6 = concatenate([Conv2DTranspose(64, (3, 3), strides=(
        2, 2), padding='same')(conv5), conv4], axis=3)
    conv6 = Conv2D(64, (3, 3), activation='relu', padding='same')(up6)
    conv6 = Conv2D(64, (3, 3), activation='relu', padding='same')(conv6)
    conv6 = Dropout(dropout)(conv6)

    up7 = concatenate([Conv2DTranspose(32, (3, 3), strides=(
        2, 2), padding='same')(conv6), conv3], axis=3)
    conv7 = Conv2D(32, (3, 3), activation='relu', padding='same')(up7)
    conv7 = Dropout(dropout)(conv7)
    conv7 = Conv2D(32, (3, 3), activation='relu', padding='same')(conv7)
    conv7 = Dropout(dropout)(conv7)

    up8 = concatenate([Conv2DTranspose(16, (3, 3), strides=(
        2, 2), padding='same')(conv7), conv2], axis=3)
    conv8 = Conv2D(16, (3, 3), activation='relu', padding='same')(up8)
    conv8 = Dropout(dropout)(conv8)
    conv8 = Conv2D(16, (3, 3), activation='relu', padding='same')(conv8)
    conv8 = Dropout(dropout)(conv8)

    up9 = concatenate([Conv2DTranspose(8, (3, 3), strides=(
        2, 2), padding='same')(conv8), conv1], axis=3)
    conv9 = Conv2D(8, (3, 3), activation='relu', padding='same')(up9)
    conv9 = Dropout(dropout)(conv9)
    conv9 = Conv2D(8, (3, 3), activation='relu', padding='same')(conv9)
    conv9 = Dropout(dropout)(conv9)

    conv10 = Conv2D(1, (1, 1), activation='sigmoid')(conv9)  # 3
    model = Model(inputs=[inputs], outputs=[conv10])

    return model
